#include <QUrl>
#include <time.h>

#include "MessagingUtils.h"
#include "Meeting.h"
#include "Configuration.h"
#include "DateTimeSettings.h"

/*
 * BaseMessage class functions
 */
BaseMessage::BaseMessage()
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::BaseMessage");
#endif
	iMessage = new QDomDocument();
#ifdef MU_DEBUG
	qDebug("BaseMessage::BaseMessage end");
#endif
}

BaseMessage::~BaseMessage()
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::~BaseMessage");
#endif
	if (iMessage)
	{
		delete iMessage;
		iMessage = 0;
	}
#ifdef MU_DEBUG
	qDebug("BaseMessage::~BaseMessage end");
#endif
}

QList<QDomNode> BaseMessage::getNodesByName(const QString& aNodeName, const QString& aParentName, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::getNodesByName");
#endif

	//TODO: Add implementation for attribute search

	QDomNodeList list;
	QList<QDomNode> specList;

	list
			= (aRootNode ) ? aRootNode->toElement().elementsByTagName(aNodeName) : iMessage->elementsByTagName(aNodeName);

	for (int i=0; i<list.count(); i++)
	{
		if (aParentName != QString::null )
		{
			if( !list.item( i ).parentNode().isNull() && list.item( i ).parentNode().isElement() && list.item( i ).parentNode().toElement().tagName().compare( aParentName ) == 0 )
			specList.append( list.item( i ) );
		}
		else
		specList.append( list.item( i ) );
	}
#ifdef MU_DEBUG
	qDebug( "BaseMessage::getNodesByName end" );
#endif		
	return specList;
}

QDomNode BaseMessage::getNodeFromDocument(const QString& aNodeName, QDomNode::NodeType aNodeType, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::getNodeFromDocument");
#endif
	QDomNode node;
	if (aNodeType == QDomNode::ElementNode)
		node
				= getElementFromDocument(aNodeName, aParentName, aIndex, aRootNode);
	else if (aNodeType == QDomNode::AttributeNode)
		node
				= getAttributeFromDocument(aNodeName, aParentName, aIndex, aRootNode);
#ifdef MU_DEBUG
	qDebug("BaseMessage::getNodeFromDocument end");
#endif
	return node;
}

QDomNode BaseMessage::getAttributeFromDocument(const QString& aAttributeName, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::getAttributeFromDocument");
#endif

	QDomNode root;
	QDomNode node;
	QDomNode parent;

	root = ( !aRootNode ) ? iMessage->documentElement() : *aRootNode;
	if (root.isNull() )
	{
#ifdef MU_DEBUG
		qDebug("BaseMessage::getAttributeFromDocument : RootNode is NULL.");
#endif		
		return node;
	}

	if (aParentName != QString::null )
	{
		if( root.isElement() )
		{
			QDomNodeList list = root.toElement().elementsByTagName( aParentName );
			if( list.count()> 0 ) parent = list.item( 0 );
		}
	}

	QDomNamedNodeMap attrs = parent.attributes();

	int count = attrs.count();
	if( count==0 || aIndex >= count || !attrs.contains( aAttributeName ) ) return root;

	QList<QDomAttr> list;

	for( int i=0; i<count; i++ )
	{
		QDomAttr attr = attrs.item( i ).toAttr();

		if( attr.name().compare( aAttributeName ) == 0 || attr.localName().compare( aAttributeName ) == 0 )
		{
			if( aParentName != QString::null && ( attr.ownerElement().tagName().compare( aParentName ) == 0 || attr.ownerElement().localName().compare( aParentName ) == 0 ))
			{
				list.append( attr );
			}
			else
			list.append( attr );
		}

	}

	if( list.count()> 0 )
	node = list.at( aIndex );

#ifdef MU_DEBUG
	qDebug( "BaseMessage::getAttributeFromDocument end." );
#endif	

	return node;
}

QDomNode BaseMessage::getElementFromDocument(const QString& aElementName, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::getElementFromDocument");
#endif
	QDomNode node;
	QDomNodeList elems;

	if ( !aRootNode)
	{
		elems = iMessage->elementsByTagName(aElementName);
	}
	else
	{
		elems = aRootNode->toElement().elementsByTagName(aElementName);
	}

	int count = elems.count();
	if (count == 0 || aIndex >= count)
	{
#ifdef MU_DEBUG
		qDebug("BaseMessage::getElementFromDocument : No Elements found.");
#endif			
		return node; //no elements found
	}

	QList<QDomElement> list;

	for (int i=0; i<count; i++)
	{
		QDomElement elem = elems.item( i ).toElement();
		if (elem.tagName().compare(aElementName) == 0 || elem.localName().compare(aElementName) == 0)
		{
			if (aParentName != QString::null )
			{
				if( !elem.parentNode().isNull() )
				{
					if( elem.parentNode().toElement().tagName().compare( aParentName ) == 0 || elem.parentNode().toElement().tagName().compare( aParentName ) == 0 )
					list.append( elem );
				}
			}
			else
			list.append( elem );
		}

	}

	if( list.count()> 0 )
	node = list.at( aIndex );

#ifdef MU_DEBUG
	qDebug( "BaseMessage::getElementFromDocument end" );
#endif		
	return node;
}

bool BaseMessage::matchName(const QDomNode& aNode, const QString& aName)
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::matchName");
#endif
	if (aNode.isElement() )
	{
		if (aNode.toElement().tagName().toLower().compare(aName.toLower() ) == 0 || aNode.localName().toLower().compare(aName.toLower() ) == 0)
		{
#ifdef MU_DEBUG
			qDebug("BaseMessage::matchName end");
#endif			
			return true;
		}
	}
	else if (aNode.isAttr() )
	{
		if (aNode.toAttr().name().toLower().compare(aName.toLower() ) == 0 || aNode.localName().toLower().compare(aName.toLower() ) == 0)
		{
#ifdef MU_DEBUG
			qDebug("BaseMessage::matchName end");
#endif			
			return true;
		}
	}
#ifdef MU_DEBUG
	qDebug("BaseMessage::matchName : No match.");
#endif	
	return false;
}

QByteArray BaseMessage::getMessage()
{
#ifdef MU_DEBUG
	qDebug("BaseMessage::getMessage");
#endif
	QByteArray msg;
	if (iMessage)
	{
		msg.append(iMessage->toByteArray() );
	}
#ifdef MU_DEBUG
	qDebug("BaseMessage::getMessage end");
#endif	
	return msg;
}

/*
 * RequestMessage class functions
 */
RequestMessage::RequestMessage(RequestCommandId aCommandId) :
	iCurrCmd(aCommandId)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::RequestMessage");
#endif
	createEnvelopeBase();

	if (aCommandId != ReqCmdNoCommand)
	{
		createMessageStructure(aCommandId);
	}
#ifdef MU_DEBUG
	qDebug("RequestMessage::RequestMessage end");
#endif
}

RequestMessage::RequestMessage(const QString& /*aFileName*/)
{
	//createEnvelopeBase();
	//TODO: read operation structures from a file
}

int RequestMessage::addNode(const QDomNode& aNode, QDomNode::NodeType aNodeType, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::addNode");
#endif

	if ( !iMessage)
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::addNode : iMessage is NULL.");
#endif	
		return MsgErrSomeError;
	}

	int err = MsgErrNoError;

	if (aNodeType == QDomNode::ElementNode && aNode.isElement() )
	{
		if (aParentName == QString::null )
		{
			if( !aRootNode )
			iMessage->appendChild( aNode );
			else
			aRootNode->appendChild( aNode );
		}
		else
		{
			QDomNodeList list = ( aRootNode ) ? aRootNode->toElement().elementsByTagName( aParentName ) : iMessage->elementsByTagName( aParentName );

			if( list.count() == 0 || aIndex >= list.count() ) err = MsgErrSomeError;
			else
			{
				QDomElement node = list.at( aIndex ).toElement();
				node.appendChild( aNode );
			}
		}
	}
	else if( aNodeType == QDomNode::AttributeNode && aNode.isAttr() )
	{
		if( aParentName != QString::null )
		{
			QDomNodeList list = ( aRootNode ) ? aRootNode->toElement().elementsByTagName( aParentName ) : iMessage->elementsByTagName( aParentName );

			if( list.count() == 0 || aIndex >= list.count() ) err = MsgErrSomeError;
			else
			{
				QDomElement node = list.at( aIndex ).toElement();
				node.setAttributeNode( aNode.toAttr() );
			}
		}
		else
		err = MsgErrSomeError; //Attribute must have a parent specified
	}
	else
	err = MsgErrSomeError;

#ifdef MU_DEBUG
	qDebug( "RequestMessage::addNode end : err=%d", err );
#endif		
	return err;
}

int RequestMessage::setNodeValue(const QString& aNodeName, const QString& aValue, QDomNode::NodeType aNodeType, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::setNodeValue");
#endif

	QDomNode
			node =
					getNodeFromDocument(aNodeName, aNodeType, aParentName, aIndex, aRootNode);
	if (node.isNull() || !iMessage)
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::setNodeValue : Node is NULL or iMessage is NULL.");
#endif		
		return MsgErrSomeError;
	}

	int err = MsgErrNoError;

	if (node.isElement() )
	{
		node.appendChild(iMessage->createTextNode(aValue) );
	}
	else if (node.isAttr() )
		node.toAttr().setValue(aValue);
	else
		err = MsgErrSomeError;

#ifdef MU_DEBUG
	qDebug("RequestMessage::setNodeValue end : err=%d", err);
#endif	
	return err;
}

int RequestMessage::createEnvelopeBase()
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::createEnvelopeBase");
#endif
	if (iMessage)
	{
		delete iMessage;
		iMessage = 0;
	}

	iMessage = new QDomDocument();

	int size = sizeof(reqCmdArrayEnvelopeBase ) / sizeof(MessageBodyElement);
	int err = MsgErrNoError;

	QDomNode base = constructArrayToNode(reqCmdArrayEnvelopeBase, size);

	if ( !base.isNull() )
		err = addNode(base);
	else
		err = MsgErrSomeError;

#ifdef MU_DEBUG
	qDebug("RequestMessage::createEnvelopeBase end : err=%d", err);
#endif		
	return err;
}

QString RequestMessage::getContentTypeForHeader(RequestCommandId aCommandId)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::getContentTypeForHeader");
#endif
	QString contentType(QString::null );

	RequestCommandId cmd = ( aCommandId == ReqCmdNoCommand ) ? iCurrCmd : aCommandId;

	if( cmd != ReqCmdNoCommand )
	{
		QString operation( QString::null );
		switch( cmd )
		{
			//Used commands
			case ReqCmdGetUserAvailability :
			{	operation = "GetUserAvailability";}break;
			case ReqCmdConvertId :
			{	operation = "ConvertId";}break;
			case ReqCmdGetItem :
			{	operation = "GetItem";}break;

			//Currently unused operations

			/*case ReqCmdAddDelegate : { operation = "AddDelegate"; } break;
			 case ReqCmdCopyFolder : { operation = "CopyFolder"; } break;
			 case ReqCmdCopyItem : { operation = "CopyItem"; } break;
			 case ReqCmdCreateAttachment : { operation = "CreateAttachment"; } break;
			 case ReqCmdCreateFolder : { operation = "CreateFolder"; } break;
			 case ReqCmdCreateItem : { operation = "CreateItem"; } break;
			 case ReqCmdCreateManagedFolder : { operation = "CreateManagedFolder"; } break;
			 case ReqCmdDeleteAttachment : { operation = "DeleteAttachment"; } break;
			 case ReqCmdDeleteFolder : { operation = "DeleteFolder"; } break;
			 case ReqCmdDeleteItem : { operation = "DeleteItem"; } break;
			 case ReqCmdExpandDL : { operation = "ExpandDL"; } break;
			 case ReqCmdFindFolder : { operation = "FindFolder"; } break;
			 case ReqCmdFindItem : { operation = "FindItem"; } break;
			 case ReqCmdGetAttachment : { operation = "GetAttachment"; } break;
			 case ReqCmdGetDelegate : { operation = "GetDelegate"; } break;
			 case ReqCmdGetEvents : { operation = "GetEvents"; } break;
			 case ReqCmdGetFolder : { operation = "GetFolder"; } break;
			 case ReqCmdGetUserOofSettings : { operation = "GetUserOofSettings"; } break;
			 case ReqCmdMoveFolder : { operation = "MoveFolder"; } break;
			 case ReqCmdMoveItem : { operation = "MoveItem"; } break;
			 case ReqCmdRemoveDelegate : { operation = "RemoveDelegate"; } break;
			 case ReqCmdResolveNames : { operation = "ResolveNames"; } break;
			 case ReqCmdSendItem : { operation = "SendItem"; } break;
			 case ReqCmdSetUserOofSettings : { operation = "SetUserOofSettings"; } break;
			 case ReqCmdSubscribe : { operation = "Subscribe"; } break;
			 case ReqCmdSyncFolderHierarchy : { operation = "SyncFolderHierarchy"; } break;
			 case ReqCmdSyncFolderItems : { operation = "SyncFolderItems"; } break;
			 case ReqCmdUnsubscribe : { operation = "Unsubscribe"; } break;
			 case ReqCmdUpdateDelegate : { operation = "UpdateDelegate"; } break;
			 case ReqCmdUpdateFolder : { operation = "UpdateFolder"; } break;
			 case ReqCmdUpdateItem : { operation = "UpdateItem"; } break;*/
			default: break;
		};

		if( operation != QString::null )
		{
			QString content( "text/xml; " );
			QString charset( "utf-8; " );
			contentType = QString( content + "charset=" + charset + "action=\"" + ACTION_URL + operation + "\"" );
		}
	}
#ifdef MU_DEBUG
	qDebug( "RequestMessage::getContentTypeForHeader end : contentType=%s", contentType.toStdString().data() );
#endif		
	return contentType;
}

int RequestMessage::createMessageStructure()
{
	return createMessageStructure(iCurrCmd);
}

int RequestMessage::createMessageStructure(RequestCommandId aCommandId)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::createMessageStructure");
#endif
	if ( !iMessage)
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::createMessageStructure : iMessage is NULL");
#endif		
		return MsgErrSomeError;
	}

	//Should always be soap:Envelope or first element in envelopebase array
	QDomNode last = iMessage->documentElement();

	if ( !matchName(last, reqCmdArrayEnvelopeBase[0].iElementName) )
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::createMessageStructure : Document element is not valid SOAP envelope");
#endif		
		return MsgErrSomeError;
	}

	QDomNode cmd;
	int err = MsgErrNoError;

	switch (aCommandId)
	{
		case ReqCmdGetUserAvailability:
		{
			cmd
					= constructArrayToNode(reqCmdArrayGetUserAvailability, sizeof(reqCmdArrayGetUserAvailability )
							/ sizeof(MessageBodyElement));
		}
			break;
		case ReqCmdConvertId:
		{
			cmd
					= constructArrayToNode(reqCmdArrayConvertId, sizeof(reqCmdArrayConvertId )
							/ sizeof(MessageBodyElement));
		}
			break;
		case ReqCmdGetItem:
		{
			cmd
					= constructArrayToNode(reqCmdArrayGetCalendarItem, sizeof(reqCmdArrayGetCalendarItem )
							/ sizeof(MessageBodyElement));
		}
			break;
		default:
			break;
	};

	if (cmd.isNull() )
		err = MsgErrSomeError;
	else
	{
		err
				= addNode(cmd, QDomNode::ElementNode, reqCmdArrayEnvelopeBase[0].iElementName);
	}
#ifdef MU_DEBUG
	qDebug("RequestMessage::createMessageStructure end : err=%d", err);
#endif	
	return err;
}

QDomNode RequestMessage::constructArrayToNode(const MessageBodyElement* aArray, int aSize)
{
#ifdef MU_DEBUG
	qDebug("RequestMessage::constructArrayToNode");
#endif
	QDomElement target; //final node to be returned

	if ( !iMessage || aSize == 0)
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::constructArrayToNode : iMessage is NULL or aSize is 0.");
#endif
		return target;
	}

	QDomElement last; // last appended node

	int currentLvl = 0; //current traversal level
	int idx = 0;

	//get the root
	MessageBodyElement root = aArray[idx];
	if (root.iNodeType != QDomNode::ElementNode || root.iTraversalLevel > 0)
	{
#ifdef MU_DEBUG
		qDebug("RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults.");
#endif		
		return target; //Root must be an element and on level 0
	}
	else
	{
		if (root.iNamespace != QString::null )
		{
			//QString prefix = root.iNamespace
			target = iMessage->createElementNS( root.iNamespace, root.iElementName );
		}
		else
		{
			target = iMessage->createElement( root.iElementName );
		}
		last = target;
		idx++;
	}

	for( int i=idx; i<aSize; i++ )
	{
		MessageBodyElement element = aArray[i];

		if( element.iNodeType == QDomNode::ElementNode )
		{
			//Node is an element node
			QDomElement elem;
			if( element.iNamespace != QString::null )
			{
				elem = iMessage->createElementNS( element.iNamespace, element.iElementName );
			}
			else
			{
				elem = iMessage->createElement( element.iElementName );
			}
			if( element.iTraversalLevel == currentLvl )
			{
				if( currentLvl == 0 )
				{
#ifdef MU_DEBUG
					qDebug( "RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults." );
#endif							
					return target; //Only root can be at traversal level 0
				}
				else
				{
					//Sibling to previous
					if( !last.parentNode().isNull() )
					last.parentNode().appendChild( elem );
					else
					{
#ifdef MU_DEBUG
						qDebug( "RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults." );
#endif	
						return target; //malformed. if last has no parent, it's a rootnode
					}
				}
			}
			else if( element.iTraversalLevel> currentLvl )
			{
				//check if node is child to previous
				//else there is something wrong in array representing the structure
				if( element.iTraversalLevel == currentLvl+1 )
				{
					last.appendChild( elem );
				}
				else
				{
#ifdef MU_DEBUG
					qDebug( "RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults." );
#endif	
					return target;
				}
			}
			else if( element.iTraversalLevel < currentLvl )
			{

				//Node is on more shallow level than previous
				//let's get it's parent
				int diff = currentLvl - element.iTraversalLevel;

				for( int j=0; j<diff; j++ )
				{
					QDomNode temp = last.parentNode();
					last = temp.toElement();
				}
				last.parentNode().appendChild( elem );
			}
			else
			{
#ifdef MU_DEBUG
				qDebug( "RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults." );
#endif				
				//some undefined error
				return target;
			}

			last = elem;
		}
		else if( element.iNodeType == QDomNode::AttributeNode )
		{
			//Node is an attribute node

			QDomAttr attr;
			if( element.iNamespace != QString::null )
			attr = iMessage->createAttributeNS( element.iNamespace, element.iElementName );
			else
			attr = iMessage->createAttribute( element.iElementName );

			if( element.iTraversalLevel == currentLvl )
			{
				last.setAttributeNode( attr );
			}
			else
			{
#ifdef MU_DEBUG
				qDebug( "RequestMessage::constructArrayToNode : Malformed message definition. Check array for faults." );
#endif					
				//error in template array.
				//attributes for element must be specified right after element
				//and must have same traversalId
				return target;
			}
		}
		else
		{
#ifdef MU_DEBUG
			qDebug( "RequestMessage::constructArrayToNode : Node type not supported." );
#endif	
			//node type not supported
			return target;
		}

		currentLvl = element.iTraversalLevel;
	}
#ifdef MU_DEBUG
	qDebug( "RequestMessage::constructArrayToNode : end" );
#endif	
	return target;
}

/*
 * ResponseMessage class functions
 */
ResponseMessage::ResponseMessage()
{
#ifdef MU_DEBUG
	qDebug("ResponseMessage::ResponseMessage");
#endif
	iMessage = new QDomDocument();
#ifdef MU_DEBUG
	qDebug("ResponseMessage::ResponseMessage end");
#endif
}

ResponseMessage::ResponseMessage(const QByteArray& aData)
{
#ifdef MU_DEBUG
	qDebug("ResponseMessage::ResponseMessage");
#endif
	iMessage->setContent(aData, true);
#ifdef MU_DEBUG
	qDebug("ResponseMessage::ResponseMessage end");
#endif
}

QString ResponseMessage::getNodeValue(const QString& aNodeName, QDomNode::NodeType aNodeType, const QString& aParentName, int aIndex, QDomNode* aRootNode)
{
#ifdef MU_DEBUG
	qDebug("ResponseMessage::getNodeValue");
#endif	
	QString value = QString::null;
	QDomNode node = getNodeFromDocument( aNodeName, aNodeType, aParentName, aIndex, aRootNode );
	if( !node.isNull() )
	{
		if( node.isElement() )
		value = node.toElement().text();
		else if( node.isAttr() )
		value = node.toAttr().value();
	}
#ifdef MU_DEBUG
	qDebug( "ResponseMessage::getNodeValue end" );
#endif	
	return value;
}

bool ResponseMessage::hasErrors()
{
#ifdef MU_DEBUG
	qDebug("ResponseMessage::hasErrors");
#endif
	if ( !iMessage)
	{
#ifdef MU_DEBUG
		qDebug("ResponseMessage::hasErrors : iMessage is NULL.");
#endif
		return true;
	}

	QDomElement root = iMessage->documentElement();
	bool err = false;

	QString rootname = (root.prefix() == QString::null ) ? root.tagName() : root.localName();
	err = ( rootname.compare( QString( "Envelope" ) ) == 0 ) ? false : true;

	QDomNodeList list = iMessage->elementsByTagName( QString( "m:ResponseCode" ) );
	for( int i=0; i<list.count(); i++ )
	{
		if( list.item(i).nodeValue().compare( "NoError" ) != 0 )
		{
			err = true;
		}
	}
#ifdef MU_DEBUG
	qDebug( "ResponseMessage::hasErrors end : err=%d", err );
#endif	
	return err;
}

/*
 * ReqMsgGetUserAvailability class functions
 */

ReqMsgGetUserAvailability::ReqMsgGetUserAvailability() :
	RequestMessage(ReqCmdGetUserAvailability)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::ReqMsgGetUserAvailability");
#endif
	if (iMessage && iCurrCmd == ReqCmdGetUserAvailability)
	{
		setNodeValue(QString("MergedFreeBusyIntervalInMinutes"), QString::number( 60) );
		setNodeValue(QString("RequestedView"), QString("DetailedMerged") );
	}
#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::ReqMsgGetUserAvailability end");
#endif	
}

int ReqMsgGetUserAvailability::addUser(const QString& aAddress, const QString& /*aAttendeeType*/, const QString& aExcludeConflicts)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::addUser");
#endif
	if ( !iMessage || iCurrCmd != ReqCmdGetUserAvailability)
	{
#ifdef MU_DEBUG
		qDebug("ReqMsgGetUserAvailability::addUser : iMessage is NULL or current command id is wrong");
#endif		
		return MsgErrSomeError;
	}

	int err = MsgErrNoError;
	int size = sizeof(reqCmdMailboxElement ) / sizeof(MessageBodyElement);

	//Create mailbox data element
	QDomNode mailbox = constructArrayToNode(reqCmdMailboxElement, size);
	if ( !mailbox.isNull() )
	{
		QDomNode email;

		if (mailbox.hasChildNodes() )
		{
			for (QDomNode subnode = mailbox.firstChild(); !subnode.isNull(); subnode
					= subnode.nextSibling() )
			{
				qDebug("subnode: %s", subnode.toElement().tagName().toStdString().data() );
				if (matchName(subnode, QString("Email") ) )
				{
					email = subnode;
				}
			}
		}

		setNodeValue(QString("Address"), aAddress, QDomNode::ElementNode, QString("Email"), 0, &email);
		setNodeValue(QString("AttendeeType"), QString("Required"), QDomNode::ElementNode, QString("MailboxData"), 0, &mailbox);
		setNodeValue(QString("ExcludeConflicts"), aExcludeConflicts, QDomNode::ElementNode, QString("MailboxData"), 0, &mailbox);

		err
				= addNode(mailbox, QDomNode::ElementNode, QString("MailboxDataArray") );
	}
	else
		err = MsgErrSomeError;

#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::addUser end : err=%d", err);
#endif	
	return err;
}

int ReqMsgGetUserAvailability::setTimeWindow(const QDateTime& aStart, const QDateTime& aEnd)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::setTimeWindow");
#endif
	if ( !iMessage || iCurrCmd != ReqCmdGetUserAvailability)
	{
#ifdef MU_DEBUG
		qDebug("ReqMsgGetUserAvailability::setTimeWindow : iMessage is NULL or command id is wrong");
#endif		
		return MsgErrSomeError;
	}

	int err = MsgErrNoError;

	err = setNodeValue(QString("StartTime"), aStart.toString(Qt::ISODate) );
	err = setNodeValue(QString("EndTime"), aEnd.toString(Qt::ISODate) );

#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::setTimeWindow end : err=%d", err);
#endif		
	return err;
}

int ReqMsgGetUserAvailability::setTimeZone()
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::setTimeZone");
#endif
	if ( !iMessage || iCurrCmd != ReqCmdGetUserAvailability)
	{
#ifdef MU_DEBUG
		qDebug("ReqMsgGetUserAvailability::setTimeZone : iMessage is NULL or command id is wrong");
#endif		
		return MsgErrSomeError;
	}
	int err = MsgErrNoError;

	time_t rawtime;
	tm *localTime;

	time(&rawtime);
	localTime = localtime(&rawtime);

	int offsetMinutes = localTime->tm_gmtoff / 60;
	if (localTime->tm_isdst)
		offsetMinutes -= 60; // If DST is in use then reduce an hour from offset, because
	// DST will be added to the offset later and it already includes
	// DST. TODO: This is silly and must be changed if possible.
	// If you can get UTC offset without DST, use it here.

#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::setTimeZone - offset in minutes=%d", offsetMinutes);
#endif

	 DateTimeSettings *dateTimeSettings = Configuration::instance()->dateTimeSettings();
	 
	 setNodeValue( QString( "Bias" ), QString::number( -offsetMinutes ), QDomNode::ElementNode, QString( "TimeZone" ) );
	 setNodeValue( QString( "Bias" ), QString::number( 0 ), QDomNode::ElementNode, QString( "StandardTime" ) );
	 setNodeValue( QString( "Time" ), dateTimeSettings->STDTime(), QDomNode::ElementNode, QString( "StandardTime" ) );
	 setNodeValue( QString( "DayOrder" ), dateTimeSettings->STDDay(), QDomNode::ElementNode, QString( "StandardTime" ) );
	 setNodeValue( QString( "Month" ), dateTimeSettings->STDMonth(), QDomNode::ElementNode, QString( "StandardTime" ) );
	 setNodeValue( QString( "DayOfWeek" ), dateTimeSettings->switchDay(), QDomNode::ElementNode, QString( "StandardTime" ) );
	 setNodeValue( QString( "Bias" ), QString::number( -60 ), QDomNode::ElementNode, QString( "DaylightTime" ) );
	 setNodeValue( QString( "Time" ), dateTimeSettings->DSTTime(), QDomNode::ElementNode, QString( "DaylightTime" ) );
	 setNodeValue( QString( "DayOrder" ), dateTimeSettings->DSTDay(), QDomNode::ElementNode, QString( "DaylightTime" ) );
	 setNodeValue( QString( "Month" ), dateTimeSettings->DSTMonth(), QDomNode::ElementNode, QString( "DaylightTime" ) );
	 setNodeValue( QString( "DayOfWeek" ), dateTimeSettings->switchDay(), QDomNode::ElementNode, QString( "DaylightTime" ) );

#ifdef MU_DEBUG
	qDebug("ReqMsgGetUserAvailability::setTimeZone end : err=%d", err);
#endif		
	return err;
}

/*
 * ReqMsgConvertMeetingId class functions
 */
ReqMsgConvertMeetingId::ReqMsgConvertMeetingId(const QString& aItemId, const QString& aMailbox)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgConvertMeetingId::ReqMsgConvertMeetingId");
#endif	
	//For this class, we're not going to provide id for base class as parameter,
	//because before content structure initialization, we must add SOAP header to envelope
	iCurrCmd = ReqCmdConvertId;

	QDomNode
			header =
					constructArrayToNode(reqCmdArrayConvertIdHeader, sizeof(reqCmdArrayConvertIdHeader )
							/ sizeof(MessageBodyElement));

	if ( !header.isNull() )
	{
		int
				err =
						addNode(header, QDomNode::ElementNode, reqCmdArrayEnvelopeBase[0].iElementName);

		if (err == MsgErrNoError)
		{
			createMessageStructure(ReqCmdConvertId);

			if (aItemId != QString::null ) setItemId( aItemId );
			if( aMailbox != QString::null ) setMailbox( aMailbox );

			setNodeValue( QString( "Version" ), QString( "Exchange2007_SP1" ), QDomNode::AttributeNode, QString( "RequestServerVersion" ) );
			setNodeValue( QString( "DestinationFormat" ), QString( "EwsLegacyId" ), QDomNode::AttributeNode, QString( "ConvertId" ) );
			setNodeValue( QString( "Format" ), QString( "HexEntryId" ), QDomNode::AttributeNode, QString( "AlternateId" ) );
		}
	}
	//setNodeValue( QString( "xmlns:t" ), NS_T, QDomNode::AttributeNode, QString( "ConvertId" ) );

	//Must be added (for now at least) like this, because Envelope is initialized already, without attributes
	/*QDomAttr attr = iMessage->createAttribute( QString( "xmlns:t" ) );
	 attr.setValue( NS_T );
	 addNode( attr, QDomNode::AttributeNode, QString( "Envelope" ) );
	 */
#ifdef MU_DEBUG
	qDebug( "ReqMsgConvertMeetingId::ReqMsgConvertMeetingId end" );
#endif		
}

int ReqMsgConvertMeetingId::setItemId(const QString& aItemId)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgConvertMeetingId::setItemId");
#endif	
	int
			err =
					setNodeValue(QString("Id"), aItemId, QDomNode::AttributeNode, QString("AlternateId") );
#ifdef MU_DEBUG
	qDebug("ReqMsgConvertMeetingId::setItemId end : err=%d", err);
#endif	
	return err;
}

int ReqMsgConvertMeetingId::setMailbox(const QString& aMailbox)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgConvertMeetingId::setMailbox");
#endif	
	int
			err =
					setNodeValue(QString("Mailbox"), aMailbox, QDomNode::AttributeNode, QString("AlternateId") );
#ifdef MU_DEBUG
	qDebug("ReqMsgConvertMeetingId::setMailbox end : err=%d", err);
#endif
	return err;
}

/*
 * ReqMsgGetCalendarItem class functions
 */
ReqMsgGetCalendarItem::ReqMsgGetCalendarItem(const QString& aItemId) :
	RequestMessage(ReqCmdGetItem)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetCalendarItem::ReqMsgGetCalendarItem");
#endif	
	if (iMessage && iCurrCmd == ReqCmdGetItem)
	{
		setNodeValue(QString("BaseShape"), QString("AllProperties") );

		if (aItemId != QString::null )
		setItemId( aItemId );
	}
#ifdef MU_DEBUG
	qDebug( "ReqMsgGetCalendarItem::ReqMsgGetCalendarItem end" );
#endif
}

int ReqMsgGetCalendarItem::setItemId(const QString& aItemId)
{
#ifdef MU_DEBUG
	qDebug("ReqMsgGetCalendarItem::setItemId");
#endif	
	int
			err =
					setNodeValue(QString("Id"), aItemId, QDomNode::AttributeNode, QString("ItemId"), 0);
#ifdef MU_DEBUG
	qDebug("ReqMsgGetCalendarItem::setItemId end : err=%d", err);
#endif	
	return err;
}

/*
 * ResMsgGetUserAvailability class functions
 */
ResMsgGetUserAvailability::ResMsgGetUserAvailability(const QByteArray& aData) :
	ResponseMessage(aData)
{
#ifdef MU_DEBUG
	qDebug("ResMsgGetUserAvailability::ResMsgGetUserAvailability");
	qDebug("ResMsgGetUserAvailability::ResMsgGetUserAvailability end");
#endif
}

int ResMsgGetUserAvailability::getMeetingsFromResponse(QList<Meeting*>& aMeetings, const Room &aRoom)
{
#ifdef MU_DEBUG
	qDebug("ResMsgGetUserAvailability::getMeetingsFromResponse");
#endif
	if ( !iMessage)
	{
#ifdef MU_DEBUG
		qDebug("ResMsgGetUserAvailability::getMeetingsFromResponse : iMessage is NULL");
#endif	
		return MsgErrSomeError;
	}

	//TODO: Clean this function a bit (see getMeetingDetailsFromResponse)

	int err = MsgErrNoError;

	QDomNodeList list = iMessage->elementsByTagName(QString("CalendarEvent") );

	for (int i=0; i<list.count(); i++)
	{
		QDomElement e = list.item( i ).toElement();
		QString tagName = (e.prefix() == QString::null ) ? e.tagName().toLower() : e.localName().toLower();

		if( !e.isNull() && tagName == "calendarevent" )
		{

			QString id, startsAt, endsAt, organizer, subject;

			for ( QDomNode subnode = e.firstChild(); !subnode.isNull(); subnode = subnode.nextSibling() )
			{
				QDomElement e = subnode.toElement();
				QString tagName = ( e.prefix() == QString::null ) ? e.tagName().toLower() : e.localName().toLower();

				if( tagName == QString( "starttime" ) )
				{
					startsAt = e.text();
				}
				else if( tagName == QString( "endtime" ) )
				{
					endsAt = e.text();
				}
				else if( tagName == QString( "calendareventdetails" ) )
				{

					for ( QDomNode detailnode = subnode.firstChild(); !detailnode.isNull(); detailnode = detailnode.nextSibling() )
					{

						QDomElement e = detailnode.toElement();
						QString tagName = ( e.prefix() == QString::null ) ? e.tagName().toLower() : e.localName().toLower();

						if( tagName == QString( "id" ) )
						{
							id = e.text();
						}

						if( tagName == QString( "subject" ) )
						{
							subject = e.text();
						}
					}
				}
			}

			QDateTime start = QDateTime::fromString( startsAt, Qt::ISODate );
			QDateTime end = QDateTime::fromString( endsAt, Qt::ISODate );

			aMeetings.append( new Meeting(
					id,
					aRoom,
					start,
					end,
					subject ) );

		}
	}
#ifdef MU_DEBUG
	qDebug( "ResMsgGetUserAvailability::getMeetingsFromResponse end : err=%d", err );
#endif		
	return err;
}

/*
 * ResMsgGetCalendarItem class functions
 */
int ResMsgGetCalendarItem::getMeetingDetailsFromResponse(Meeting& aMeeting)
{
#ifdef MU_DEBUG
	qDebug("ResMsgGetCalendarItem::getMeetingDetailsFromResponse");
#endif
	if ( !iMessage || hasErrors() )
	{
#ifdef MU_DEBUG
		qDebug("ResMsgGetCalendarItem::getMeetingDetailsFromResponse : iMessage is NULL or response has errors.");
#endif		
		return MsgErrSomeError;
	}

	QString name, address, description; //Organizer

	QDomNode box =
			getElementFromDocument(QString("Mailbox"), QString("Organizer"), 0);

	if ( !box.isNull() && box.hasChildNodes() )
	{
		for (QDomNode subnode = box.firstChild(); !subnode.isNull(); subnode
				= subnode.nextSibling() )
		{
			if (matchName(subnode, QString("Name") ) )
				name = subnode.toElement().text();
			else if (matchName(subnode, QString("EmailAddress") ) )
				address = subnode.toElement().text();
		}
	}

	QDomNode body =
			getElementFromDocument(QString("Body"), QString("CalendarItem"), 0);

	if (body.isElement() )
		description = body.toElement().text();

	aMeeting.setOrganizer(name, address);
	aMeeting.setDescription(description);

#ifdef MU_DEBUG
	qDebug("ResMsgGetCalendarItem::getMeetingDetailsFromResponse end");
#endif		
	return MsgErrNoError;
}

