#ifndef PROGRESSBAR_H_
#define PROGRESSBAR_H_

#include <QDialog>

class QProgressBar;
class QLabel;
class QPushButton;

//! UserInterface class.
/*!
 * UserInterface class which launches a modal progress bar during application events that might take
 * some time
 */
class ProgressBar : public QDialog
{
	Q_OBJECT

public:
	//! Constructor.
	/*!
	 * Constructor to initialize the StatusWidget instance.
	 * \param aParent Parent object.
	 * \param aTitle Title string to be set to the progress bar.
	 */
	ProgressBar( const QString &aTitle, bool aCancellable = false, QWidget *aParent = 0 );
	//! Destructor.
	virtual ~ProgressBar();
	//! Updates the text fields of progress bar dialog.
	/*!
	 * Updates the text fields of progress bar dialog.
	 * \param aMessage The text for the label of the dialog.
	 * \param aTitle Title string to be set to the progress bar.
	 */
	void update( const QString &aMessage, const QString &aTitle = "" );
	
	void toggleCancellable( bool aEnable );

signals:
	//! Signal. Emitted if user presses cancel button.
	/*!
	 * Signal. Emitted if user presses cancel button.
	 */
	void cancel();
	void started();

private:
	QProgressBar *iProgress;
	QLabel *iLabel;
	QPushButton *iButton;

};

#endif /*PROGRESSBAR_H_*/
